@@ -1,18 +1,16 @@
-// =================================================================
-//          #     #                 #     #
-//          ##    #   ####   #####  ##    #  ######   #####
-//          # #   #  #    #  #    # # #   #  #          #
-//          #  #  #  #    #  #    # #  #  #  #####      #
-//          #   # #  #    #  #####  #   # #  #          #
-//          #    ##  #    #  #   #  #    ##  #          #
-//          #     #   ####   #    # #     #  ######     #
+// ==========================================================================
+//     _   _ _ ____            ____          _____
+//    | | | (_)  _ \ ___ _ __ / ___|___  _ _|_   _| __ __ _  ___ ___ _ __
+//    | |_| | | |_) / _ \ '__| |   / _ \| '_ \| || '__/ _` |/ __/ _ \ '__|
+//    |  _  | |  __/  __/ |  | |__| (_) | | | | || | | (_| | (_|  __/ |
+//    |_| |_|_|_|   \___|_|   \____\___/|_| |_|_||_|  \__,_|\___\___|_|
 //
-//       ---   The NorNet Testbed for Multi-Homed Systems  ---
-//                       https://www.nntb.no
-// =================================================================
+//       ---  High-Performance Connectivity Tracer (HiPerConTracer)  ---
+//                 https://www.nntb.no/~dreibh/hipercontracer/
+// ==========================================================================
 //
 // High-Performance Connectivity Tracer (HiPerConTracer)
-// Copyright (C) 2015-2022 by Thomas Dreibholz
+// Copyright (C) 2015-2025 by Thomas Dreibholz
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,7 +37,9 @@
 #include <netinet/ip_icmp.h>
 
 #include <istream>
-#include <algorithm>
+#include <ostream>
+
+#include "internet16.h"
 
 
 // ==========================================================================
@@ -60,23 +60,6 @@
 class ICMPHeader
 {
    public:
-   enum {
-      IPv4EchoRequest           = ICMP_ECHO,
-      IPv4EchoReply             = ICMP_ECHOREPLY,
-      IPv4TimeExceeded          = ICMP_TIMXCEED,
-      IPv4Unreachable           = ICMP_UNREACH,
-
-      IPv6EchoRequest           = ICMP6_ECHO_REQUEST,
-      IPv6EchoReply             = ICMP6_ECHO_REPLY,
-      IPv6TimeExceeded          = ICMP6_TIME_EXCEEDED,
-      IPv6Unreachable           = ICMP6_DST_UNREACH,
-
-      IPv6NeighborSolicitation  = ND_NEIGHBOR_SOLICIT,
-      IPv6NeighborAdvertisement = ND_NEIGHBOR_ADVERT,
-      IPv6RouterSolicitation    = ND_ROUTER_SOLICIT,
-      IPv6RouterAdvertisement   = ND_ROUTER_ADVERT
-   };
-
    ICMPHeader() {
       std::fill(Data, Data + sizeof(Data), 0);
    }
@@ -84,8 +67,8 @@ class ICMPHeader
       memcpy(Data, inputData, std::min(length, (size_t)8));
    }
 
-   inline uint8_t type()        const { return Data[0];      }
-   inline uint8_t code()        const { return Data[1];      }
+   inline uint8_t  type()       const { return Data[0];      }
+   inline uint8_t  code()       const { return Data[1];      }
    inline uint16_t checksum()   const { return decode(2, 3); }
    inline uint16_t identifier() const { return decode(4, 5); }
    inline uint16_t seqNumber()  const { return decode(6, 7); }
@@ -96,48 +79,36 @@ class ICMPHeader
    inline void identifier(uint16_t id)     { encode(4, 5, id);       }
    inline void seqNumber(uint16_t seqNum)  { encode(6, 7, seqNum);   }
 
+   inline void computeInternet16(uint32_t& sum) const {
+      ::computeInternet16(sum, (uint8_t*)&Data, sizeof(Data));
+   }
+
+   inline const uint8_t* data() const {
+      return (const uint8_t*)&Data;
+   }
+   inline size_t size() const {
+      return sizeof(Data);
+   }
+
    inline friend std::istream& operator>>(std::istream& is, ICMPHeader& header) {
-      return(is.read(reinterpret_cast<char*>(header.Data), 8));
+      return is.read(reinterpret_cast<char*>(header.Data), 8);
    }
 
    inline friend std::ostream& operator<<(std::ostream& os, const ICMPHeader& header) {
-      return(os.write(reinterpret_cast<const char*>(header.Data), 8));
+      return os.write(reinterpret_cast<const char*>(header.Data), 8);
    }
 
    private:
    inline uint16_t decode(const unsigned int a, const unsigned int b) const {
-      return(((uint16_t)Data[a] << 8) + Data[b]);
+      return ((uint16_t)Data[a] << 8) + Data[b];
    }
 
    inline void encode(const unsigned int a, const unsigned int b, const uint16_t n) {
       Data[a] = static_cast<uint8_t>(n >> 8);
-      Data[b] = static_cast<uint8_t>(n & 0xFF);
+      Data[b] = static_cast<uint8_t>(n & 0xff);
    }
 
    uint8_t Data[8];
 };
 
-
-// Internet-16 checksum according to RFC 1071
-template <typename Iterator> void computeInternet16(ICMPHeader& header, Iterator bodyBegin, Iterator bodyEnd)
-{
-   uint32_t sum =
-      (header.type() << 8) +
-      header.code() +
-      header.identifier() +
-      header.seqNumber();
-
-   Iterator body_iter = bodyBegin;
-   while (body_iter != bodyEnd) {
-      sum += (static_cast<uint8_t>(*body_iter++) << 8);
-      if (body_iter != bodyEnd) {
-         sum += static_cast<uint8_t>(*body_iter++);
-      }
-   }
-
-   sum = (sum >> 16) + (sum & 0xFFFF);
-   sum += (sum >> 16);
-   header.checksum(static_cast<uint16_t>(~sum));
-}
-
 #endif
